from pandas import DataFrame, Series


def get_first_solution(n, data):
    distance_sum = DataFrame(columns=["point", "distance"])
    for element in range(n):
        element_df = data.query(f"source == {element} or destination == {element}")
        distance = element_df["distance"].sum()
        distance_sum = distance_sum.append(
            {"point": element, "distance": distance}, ignore_index=True
        )
    furthest_index = distance_sum["distance"].astype(float).idxmax()
    furthest_row = distance_sum.iloc[furthest_index]
    furthest_row["distance"] = 0
    return furthest_row


def get_different_element(original, row):
    if row.source == original:
        return row.destination
    return row.source


def get_closest_element(element, data):
    element_df = data.query(f"source == {element} or destination == {element}")
    closest_index = element_df["distance"].astype(float).idxmin()
    closest_row = data.loc[closest_index]
    closest_point = get_different_element(original=element, row=closest_row)
    return Series(data={"point": closest_point, "distance": closest_row["distance"]})


def explore_solutions(solutions, data):
    closest_elements = solutions["point"].apply(func=get_closest_element, data=data)
    furthest_index = closest_elements["distance"].astype(float).idxmax()
    return closest_elements.iloc[furthest_index]


def remove_duplicates(current, previous, data):
    duplicate_free_df = data.query(
        f"(source != {current} or destination not in @previous) and (source not in @previous or destination != {current})"
    )
    return duplicate_free_df


def greedy_algorithm(n, m, data):
    solutions = DataFrame(columns=["point", "distance"])
    first_solution = get_first_solution(n, data)
    solutions = solutions.append(first_solution, ignore_index=True)
    for _ in range(m):
        element = explore_solutions(solutions, data)
        solutions = solutions.append(element)
        data = remove_duplicates(
            current=element["point"], previous=solutions["point"], data=data
        )
    return solutions
